#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include <time.h>
#include <math.h>
#include "wgif.h"

#define WIDTH 80
#define HEIGHT 40


int fwr(uint8_t *ptr, size_t bytes, void *wp) {
    return fwrite(ptr, 1, bytes, wp);
}

int gbham_sgn(int x) {
    if (x>0) {
        return 1;
    }
    if (x<0) {
        return -1;
    }
    return 0;
}

void gbham(uint8_t *bitmap, uint8_t colorid, int x0, int y0, int x1, int y1) {
    int i;
    int x, y;
    int dx, dy;
    int incx, incy;
    int pdx, pdy, ddx, ddy;
    int dfast, dslow;
    int err;

    dx=x1-x0;
    dy=y1-y0;
    incx=gbham_sgn(dx);
    incy=gbham_sgn(dy);
    if (dx<0) {
        dx=-dx;
    }
    if (dy<0) {
        dy=-dy;
    }
    if (dx>dy) {
        pdx=incx;
        pdy=0;
        ddx=incx;
        ddy=incy;
        dslow=dy;
        dfast=dx;

    } else {
        pdx=0;
        pdy=incy;
        ddx=incx;
        ddy=incy;
        dslow=dx;
        dfast=dy;
    }
    x=x0;
    y=y0;
    err=dfast/2;
    bitmap[x+y*WIDTH]=colorid;
    for (i=0;i<dfast;i++) {
        err=err-dslow;
        if (err<0) {
            err=err+dfast;
            x=x+ddx;
            y=y+ddy;
        } else {
            x=x+pdx;
            y=y+pdy;
        }
        bitmap[x+y*WIDTH]=colorid;
    }
}

void line(struct gif_file *gf, uint8_t *bitmap, int tx, int ty) {
    uint32_t ct[2];
    
    ct[0]=(rand()&0xfff)|((rand()&0xfff)<<12);
    ct[1]=0;

    memset(bitmap, 1, WIDTH*HEIGHT);
    gif_graphicControlExtension(gf, 10, 1);
    gbham(bitmap, 0, WIDTH>>1, HEIGHT>>1, tx, ty);
    gif_image(gf, 0, 0, WIDTH, HEIGHT, ct, 2, bitmap);
}

int main() {
    struct gif_file gf;
    uint8_t bitmap[WIDTH*HEIGHT];
    int i;

    gf.wr_fnct=fwr;
    gf.wparm=fopen("bresenham.gif", "wb");
    srand(time(NULL));
    
    gif_init(&gf, WIDTH, HEIGHT, NULL, 0, 0);
    gif_ext_loop(&gf, 0);
    memset(bitmap, 0, WIDTH*HEIGHT);


    for (i=0;i<WIDTH;i++) {
        line(&gf, bitmap, i, 0);
    }

    for (i=0;i<HEIGHT;i++) {
        line(&gf, bitmap, WIDTH-1, i);
    }

    for (i=WIDTH-1;i>=0;i--) {
        line(&gf, bitmap, i, HEIGHT-1);
    }

    for (i=HEIGHT-1;i>=0;i--) {
        line(&gf, bitmap, 0, i);
    }
    gif_end(&gf);
    fclose(gf.wparm);

    return 0;
}